@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require 'multi_json'
 require 'set'
 
@@ -124,7 +126,26 @@ module Grape
     # This method is the primary means by which you will declare what attributes
     # should be exposed by the entity.
     #
+    # @option options :expose_nil When set to false the associated exposure will not
+    #   be rendered if its value is nil.
+    #
     # @option options :as Declare an alias for the representation of this attribute.
+    #   If a proc is presented it is evaluated in the context of the entity so object
+    #   and the entity methods are available to it.
+    #
+    # @example as: a proc or lambda
+    #
+    #   object = OpenStruct(awesomness: 'awesome_key', awesome: 'not-my-key', other: 'other-key' )
+    #
+    #   class MyEntity < Grape::Entity
+    #     expose :awesome, as: proc { object.awesomeness }
+    #     expose :awesomeness, as: ->(object, opts) { object.other }
+    #   end
+    #
+    #   => { 'awesome_key': 'not-my-key', 'other-key': 'awesome_key' }
+    #
+    # Note the parameters passed in via the lambda syntax.
+    #
     # @option options :if When passed a Hash, the attribute will only be exposed if the
     #   runtime options match all the conditions passed in. When passed a lambda, the
     #   lambda will execute with two arguments: the object being represented and the
@@ -151,11 +172,12 @@ module Grape
       options = merge_options(args.last.is_a?(Hash) ? args.pop : {})
 
       if args.size > 1
-        fail ArgumentError, 'You may not use the :as option on multi-attribute exposures.' if options[:as]
-        fail ArgumentError, 'You may not use block-setting on multi-attribute exposures.' if block_given?
+        raise ArgumentError, 'You may not use the :as option on multi-attribute exposures.' if options[:as]
+        raise ArgumentError, 'You may not use the :expose_nil on multi-attribute exposures.' if options.key?(:expose_nil)
+        raise ArgumentError, 'You may not use block-setting on multi-attribute exposures.' if block_given?
       end
 
-      fail ArgumentError, 'You may not use block-setting when also using format_with' if block_given? && options[:format_with].respond_to?(:call)
+      raise ArgumentError, 'You may not use block-setting when also using format_with' if block_given? && options[:format_with].respond_to?(:call)
 
       if block_given?
         if block.parameters.any?
@@ -167,24 +189,24 @@ module Grape
 
       @documentation = nil
       @nesting_stack ||= []
+      args.each { |attribute| build_exposure_for_attribute(attribute, @nesting_stack, options, block) }
+    end
 
-      # rubocop:disable Style/Next
-      args.each do |attribute|
-        exposure = Exposure.new(attribute, options)
+    def self.build_exposure_for_attribute(attribute, nesting_stack, options, block)
+      exposure_list = nesting_stack.empty? ? root_exposures : nesting_stack.last.nested_exposures
 
-        if @nesting_stack.empty?
-          root_exposures << exposure
-        else
-          @nesting_stack.last.nested_exposures << exposure
-        end
+      exposure = Exposure.new(attribute, options)
 
-        # Nested exposures are given in a block with no parameters.
-        if exposure.nesting?
-          @nesting_stack << exposure
-          block.call
-          @nesting_stack.pop
-        end
-      end
+      exposure_list.delete_by(attribute) if exposure.override?
+
+      exposure_list << exposure
+
+      # Nested exposures are given in a block with no parameters.
+      return unless exposure.nesting?
+
+      nesting_stack << exposure
+      block.call
+      nesting_stack.pop
     end
 
     # Returns exposures that have been declared for this Entity on the top level.
@@ -214,7 +236,7 @@ module Grape
     end
 
     def self.cannot_unexpose!
-      fail "You cannot call 'unexpose` inside of nesting exposure!"
+      raise "You cannot call 'unexpose` inside of nesting exposure!"
     end
 
     # Set options that will be applied to any exposures declared inside the block.
@@ -237,9 +259,7 @@ module Grape
     # #docmentation, any exposure without a documentation key will be ignored.
     def self.documentation
       @documentation ||= root_exposures.each_with_object({}) do |exposure, memo|
-        if exposure.documentation && !exposure.documentation.empty?
-          memo[exposure.key] = exposure.documentation
-        end
+        memo[exposure.key] = exposure.documentation if exposure.documentation && !exposure.documentation.empty?
       end
     end
 
@@ -270,7 +290,7 @@ module Grape
     #   end
     #
     def self.format_with(name, &block)
-      fail ArgumentError, 'You must pass a block for formatters' unless block_given?
+      raise ArgumentError, 'You must pass a block for formatters' unless block_given?
       formatters[name.to_sym] = block
     end
 
@@ -392,8 +412,9 @@ module Grape
     # @option options :only [Array] all the fields that should be returned
     # @option options :except [Array] all the fields that should not be returned
     def self.represent(objects, options = {})
-      if objects.respond_to?(:to_ary) && ! @present_collection
-        root_element =  root_element(:collection_root)
+      @present_collection ||= nil
+      if objects.respond_to?(:to_ary) && !@present_collection
+        root_element = root_element(:collection_root)
         inner = objects.to_ary.map { |object| new(object, options.reverse_merge(collection: true)).presented }
       else
         objects = { @collection_name => objects } if @present_collection
@@ -409,8 +430,9 @@ module Grape
     # This method returns the entity's root or collection root node, or its parent's
     # @param root_type: either :collection_root or just :root
     def self.root_element(root_type)
-      if instance_variable_get("@#{root_type}")
-        instance_variable_get("@#{root_type}")
+      instance_variable = "@#{root_type}"
+      if instance_variable_defined?(instance_variable) && instance_variable_get(instance_variable)
+        instance_variable_get(instance_variable)
       elsif superclass.respond_to? :root_element
         superclass.root_element(root_type)
       end
@@ -424,14 +446,16 @@ module Grape
       end
     end
 
+    # Prevent default serialization of :options or :delegator.
+    def inspect
+      fields = serializable_hash.map { |k, v| "#{k}=#{v}" }
+      "#<#{self.class.name}:#{object_id} #{fields.join(' ')}>"
+    end
+
     def initialize(object, options = {})
       @object = object
-      @delegator = Delegator.new object
-      @options = if options.is_a? Options
-                   options
-                 else
-                   Options.new options
-                 end
+      @delegator = Delegator.new(object)
+      @options = options.is_a?(Options) ? options : Options.new(options)
     end
 
     def root_exposures
@@ -466,7 +490,11 @@ module Grape
     end
 
     def exec_with_object(options, &block)
-      instance_exec(object, options, &block)
+      if block.parameters.count == 1
+        instance_exec(object, &block)
+      else
+        instance_exec(object, options, &block)
+      end
     end
 
     def exec_with_attribute(attribute, &block)
@@ -478,14 +506,14 @@ module Grape
     end
 
     def delegate_attribute(attribute)
-      if respond_to?(attribute, true)
+      if respond_to?(attribute, true) && Grape::Entity > method(attribute).owner
         send(attribute)
       else
         delegator.delegate(attribute)
       end
     end
 
-    alias_method :as_json, :serializable_hash
+    alias as_json serializable_hash
 
     def to_json(options = {})
       options = options.to_h if options && options.respond_to?(:to_h)
@@ -498,8 +526,8 @@ module Grape
     end
 
     # All supported options.
-    OPTIONS = [
-      :rewrite, :as, :if, :unless, :using, :with, :proc, :documentation, :format_with, :safe, :attr_path, :if_extras, :unless_extras, :merge
+    OPTIONS = %i[
+      rewrite as if unless using with proc documentation format_with safe attr_path if_extras unless_extras merge expose_nil override
     ].to_set.freeze
 
     # Merges the given options with current block options.
@@ -509,7 +537,7 @@ module Grape
       opts = {}
 
       merge_logic = proc do |key, existing_val, new_val|
-        if [:if, :unless].include?(key)
+        if %i[if unless].include?(key)
           if existing_val.is_a?(Hash) && new_val.is_a?(Hash)
             existing_val.merge(new_val)
           elsif new_val.is_a?(Hash)
@@ -535,8 +563,8 @@ module Grape
     #
     # @param options [Hash] Exposure options.
     def self.valid_options(options)
-      options.keys.each do |key|
-        fail ArgumentError, "#{key.inspect} is not a valid option." unless OPTIONS.include?(key)
+      options.each_key do |key|
+        raise ArgumentError, "#{key.inspect} is not a valid option." unless OPTIONS.include?(key)
       end
 
       options[:using] = options.delete(:with) if options.key?(:with)
