@@ -1,8 +1,10 @@
+# frozen_string_literal: true
+
 module Grape
   class Entity
     module Exposure
       class Base
-        attr_reader :attribute, :key, :is_safe, :documentation, :conditions, :for_merge
+        attr_reader :attribute, :is_safe, :documentation, :override, :conditions, :for_merge
 
         def self.new(attribute, options, conditions, *args, &block)
           super(attribute, options, conditions).tap { |e| e.setup(*args, &block) }
@@ -11,11 +13,13 @@ module Grape
         def initialize(attribute, options, conditions)
           @attribute = attribute.try(:to_sym)
           @options = options
-          @key = (options[:as] || attribute).try(:to_sym)
+          key = options[:as] || attribute
+          @key = key.respond_to?(:to_sym) ? key.to_sym : key
           @is_safe = options[:safe]
           @for_merge = options[:merge]
           @attr_path_proc = options[:attr_path]
           @documentation = options[:documentation]
+          @override = options[:override]
           @conditions = conditions
         end
 
@@ -34,15 +38,14 @@ module Grape
             @conditions == other.conditions
         end
 
-        def setup
-        end
+        def setup; end
 
         def nesting?
           false
         end
 
         # if we have any nesting exposures with the same name.
-        def deep_complex_nesting?
+        def deep_complex_nesting?(entity) # rubocop:disable Lint/UnusedMethodArgument
           false
         end
 
@@ -51,19 +54,19 @@ module Grape
           if @is_safe
             is_delegatable
           else
-            is_delegatable || fail(NoMethodError, "#{entity.class.name} missing attribute `#{@attribute}' on #{entity.object}")
+            is_delegatable || raise(NoMethodError, "#{entity.class.name} missing attribute `#{@attribute}' on #{entity.object}")
           end
         end
 
         def value(_entity, _options)
-          fail NotImplementedError
+          raise NotImplementedError
         end
 
         def serializable_value(entity, options)
           partial_output = valid_value(entity, options)
 
           if partial_output.respond_to?(:serializable_hash)
-            partial_output.serializable_hash(options)
+            partial_output.serializable_hash
           elsif partial_output.is_a?(Array) && partial_output.all? { |o| o.respond_to?(:serializable_hash) }
             partial_output.map(&:serializable_hash)
           elsif partial_output.is_a?(Hash)
@@ -103,6 +106,10 @@ module Grape
           end
         end
 
+        def key(entity = nil)
+          @key.respond_to?(:call) ? entity.exec_with_object(@options, &@key) : @key
+        end
+
         def with_attr_path(entity, options)
           path_part = attr_path(entity, options)
           options.with_attr_path(path_part) do
@@ -110,6 +117,10 @@ module Grape
           end
         end
 
+        def override?
+          @override
+        end
+
         protected
 
         attr_reader :options
