@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require 'spec_helper'
 require 'ostruct'
 
@@ -27,7 +29,9 @@ describe Grape::Entity do
         end
 
         it 'makes sure that :format_with as a proc cannot be used with a block' do
-          expect { subject.expose :name, format_with: proc {} {} }.to raise_error ArgumentError
+          # rubocop:disable Style/BlockDelimiters
+          expect { subject.expose :name, format_with: proc {} do p 'hi' end }.to raise_error ArgumentError
+          # rubocop:enable Style/BlockDelimiters
         end
 
         it 'makes sure unknown options are not silently ignored' do
@@ -50,6 +54,142 @@ describe Grape::Entity do
           subject.expose(:special, merge: ->(_, v1, v2) { v1 && v2 ? 'brand new val' : v2 })
           expect(subject.represent(nested_hash).serializable_hash).to eq(like_nested_hash: 'brand new val')
         end
+
+        context 'and nested object is nil' do
+          let(:nested_hash) do
+            { something: nil, special: { like_nested_hash: '12' } }
+          end
+
+          it 'adds nothing to output' do
+            subject.expose(:something, merge: true)
+            subject.expose(:special)
+            expect(subject.represent(nested_hash).serializable_hash).to eq(special: { like_nested_hash: '12' })
+          end
+        end
+      end
+
+      context 'with :expose_nil option' do
+        let(:a) { nil }
+        let(:b) { nil }
+        let(:c) { 'value' }
+
+        context 'when model is a PORO' do
+          let(:model) { Model.new(a, b, c) }
+
+          before do
+            stub_const 'Model', Class.new
+            Model.class_eval do
+              attr_accessor :a, :b, :c
+
+              def initialize(a, b, c)
+                @a = a
+                @b = b
+                @c = c
+              end
+            end
+          end
+
+          context 'when expose_nil option is not provided' do
+            it 'exposes nil attributes' do
+              subject.expose(:a)
+              subject.expose(:b)
+              subject.expose(:c)
+              expect(subject.represent(model).serializable_hash).to eq(a: nil, b: nil, c: 'value')
+            end
+          end
+
+          context 'when expose_nil option is true' do
+            it 'exposes nil attributes' do
+              subject.expose(:a, expose_nil: true)
+              subject.expose(:b, expose_nil: true)
+              subject.expose(:c)
+              expect(subject.represent(model).serializable_hash).to eq(a: nil, b: nil, c: 'value')
+            end
+          end
+
+          context 'when expose_nil option is false' do
+            it 'does not expose nil attributes' do
+              subject.expose(:a, expose_nil: false)
+              subject.expose(:b, expose_nil: false)
+              subject.expose(:c)
+              expect(subject.represent(model).serializable_hash).to eq(c: 'value')
+            end
+
+            it 'is only applied per attribute' do
+              subject.expose(:a, expose_nil: false)
+              subject.expose(:b)
+              subject.expose(:c)
+              expect(subject.represent(model).serializable_hash).to eq(b: nil, c: 'value')
+            end
+
+            it 'raises an error when applied to multiple attribute exposures' do
+              expect { subject.expose(:a, :b, :c, expose_nil: false) }.to raise_error ArgumentError
+            end
+          end
+        end
+
+        context 'when model is a hash' do
+          let(:model) { { a: a, b: b, c: c } }
+
+          context 'when expose_nil option is not provided' do
+            it 'exposes nil attributes' do
+              subject.expose(:a)
+              subject.expose(:b)
+              subject.expose(:c)
+              expect(subject.represent(model).serializable_hash).to eq(a: nil, b: nil, c: 'value')
+            end
+          end
+
+          context 'when expose_nil option is true' do
+            it 'exposes nil attributes' do
+              subject.expose(:a, expose_nil: true)
+              subject.expose(:b, expose_nil: true)
+              subject.expose(:c)
+              expect(subject.represent(model).serializable_hash).to eq(a: nil, b: nil, c: 'value')
+            end
+          end
+
+          context 'when expose_nil option is false' do
+            it 'does not expose nil attributes' do
+              subject.expose(:a, expose_nil: false)
+              subject.expose(:b, expose_nil: false)
+              subject.expose(:c)
+              expect(subject.represent(model).serializable_hash).to eq(c: 'value')
+            end
+
+            it 'is only applied per attribute' do
+              subject.expose(:a, expose_nil: false)
+              subject.expose(:b)
+              subject.expose(:c)
+              expect(subject.represent(model).serializable_hash).to eq(b: nil, c: 'value')
+            end
+
+            it 'raises an error when applied to multiple attribute exposures' do
+              expect { subject.expose(:a, :b, :c, expose_nil: false) }.to raise_error ArgumentError
+            end
+          end
+        end
+
+        context 'with nested structures' do
+          let(:model) { { a: a, b: b, c: { d: nil, e: nil, f: { g: nil, h: nil } } } }
+
+          context 'when expose_nil option is false' do
+            it 'does not expose nil attributes' do
+              subject.expose(:a, expose_nil: false)
+              subject.expose(:b)
+              subject.expose(:c) do
+                subject.expose(:d, expose_nil: false)
+                subject.expose(:e)
+                subject.expose(:f) do
+                  subject.expose(:g, expose_nil: false)
+                  subject.expose(:h)
+                end
+              end
+
+              expect(subject.represent(model).serializable_hash).to eq(b: nil, c: { e: nil, f: { h: nil } })
+            end
+          end
+        end
       end
 
       context 'with a block' do
@@ -100,6 +240,30 @@ describe Grape::Entity do
           end
         end
 
+        context 'with block passed via &' do
+          it 'with does not pass options when block is passed via &' do
+            class SomeObject
+              def method_without_args
+                'result'
+              end
+            end
+
+            subject.expose :that_method_without_args do |object|
+              object.method_without_args
+            end
+
+            subject.expose :that_method_without_args_again, &:method_without_args
+
+            object = SomeObject.new
+
+            value = subject.represent(object).value_for(:that_method_without_args)
+            expect(value).to eq('result')
+
+            value2 = subject.represent(object).value_for(:that_method_without_args_again)
+            expect(value2).to eq('result')
+          end
+        end
+
         context 'with no parameters passed to the block' do
           it 'adds a nested exposure' do
             subject.expose :awesome do
@@ -119,7 +283,7 @@ describe Grape::Entity do
             expect(another_nested).to_not be_nil
             expect(another_nested.using_class_name).to eq('Awesome')
             expect(moar_nested).to_not be_nil
-            expect(moar_nested.key).to eq(:weee)
+            expect(moar_nested.key(subject)).to eq(:weee)
           end
 
           it 'represents the exposure as a hash of its nested.root_exposures' do
@@ -277,8 +441,7 @@ describe Grape::Entity do
             end
 
             additional_hash = { users: [{ id: 1, name: 'John' }, { id: 2, name: 'Jay' }],
-                                admins: [{ id: 3, name: 'Jack' }, { id: 4, name: 'James' }]
-                              }
+                                admins: [{ id: 3, name: 'Jack' }, { id: 4, name: 'James' }] }
             expect(subject.represent(additional_hash).serializable_hash).to eq(
               profiles: additional_hash[:users] + additional_hash[:admins],
               awesome: { just_a_key: 'value', just_another_key: 'value' }
@@ -313,6 +476,24 @@ describe Grape::Entity do
           expect(subject.represent({ name: 'bar' }, serializable: true)).to eq(email: nil, name: 'bar')
           expect(child_class.represent({ name: 'bar' }, serializable: true)).to eq(email: nil, name: 'foo')
         end
+
+        it 'not overrides exposure by default' do
+          subject.expose :name
+          child_class = Class.new(subject)
+          child_class.expose :name, as: :child_name
+
+          expect(subject.represent({ name: 'bar' }, serializable: true)).to eq(name: 'bar')
+          expect(child_class.represent({ name: 'bar' }, serializable: true)).to eq(name: 'bar', child_name: 'bar')
+        end
+
+        it 'overrides parent class exposure when option is specified' do
+          subject.expose :name
+          child_class = Class.new(subject)
+          child_class.expose :name, as: :child_name, override: true
+
+          expect(subject.represent({ name: 'bar' }, serializable: true)).to eq(name: 'bar')
+          expect(child_class.represent({ name: 'bar' }, serializable: true)).to eq(child_name: 'bar')
+        end
       end
 
       context 'register formatters' do
@@ -342,20 +523,20 @@ describe Grape::Entity do
 
           subject.expose :birthday, format_with: :timestamp
 
-          model  = { birthday: Time.gm(2012, 2, 27) }
+          model = { birthday: Time.gm(2012, 2, 27) }
           expect(subject.new(double(model)).as_json[:birthday]).to eq '02/27/2012'
         end
 
         it 'formats an exposure with a :format_with lambda that returns a value from the entity instance' do
           object = {}
 
-          subject.expose(:size, format_with: ->(_value) { self.object.class.to_s })
+          subject.expose(:size, format_with: ->(_value) { object.class.to_s })
           expect(subject.represent(object).value_for(:size)).to eq object.class.to_s
         end
 
         it 'formats an exposure with a :format_with symbol that returns a value from the entity instance' do
           subject.format_with :size_formatter do |_date|
-            self.object.class.to_s
+            object.class.to_s
           end
 
           object = {}
@@ -366,7 +547,7 @@ describe Grape::Entity do
 
         it 'works global on Grape::Entity' do
           Grape::Entity.format_with :size_formatter do |_date|
-            self.object.class.to_s
+            object.class.to_s
           end
           object = {}
 
@@ -485,7 +666,7 @@ describe Grape::Entity do
         end
 
         exposure = subject.find_exposure(:awesome_thing)
-        expect(exposure.key).to eq :extra_smooth
+        expect(exposure.key(subject)).to eq :extra_smooth
       end
 
       it 'merges nested :if option' do
@@ -585,6 +766,34 @@ describe Grape::Entity do
         exposure = subject.find_exposure(:awesome_thing)
         expect(exposure.documentation).to eq(desc: 'Other description.')
       end
+
+      it 'propagates expose_nil option' do
+        subject.class_eval do
+          with_options(expose_nil: false) do
+            expose :awesome_thing
+          end
+        end
+
+        exposure = subject.find_exposure(:awesome_thing)
+        expect(exposure.conditions[0].inversed?).to be true
+        expect(exposure.conditions[0].block.call(awesome_thing: nil)).to be true
+      end
+
+      it 'overrides nested :expose_nil option' do
+        subject.class_eval do
+          with_options(expose_nil: true) do
+            expose :awesome_thing, expose_nil: false
+            expose :other_awesome_thing
+          end
+        end
+
+        exposure = subject.find_exposure(:awesome_thing)
+        expect(exposure.conditions[0].inversed?).to be true
+        expect(exposure.conditions[0].block.call(awesome_thing: nil)).to be true
+        # Conditions are only added for exposures that do not expose nil
+        exposure = subject.find_exposure(:other_awesome_thing)
+        expect(exposure.conditions[0]).to be_nil
+      end
     end
 
     describe '.represent' do
@@ -597,14 +806,14 @@ describe Grape::Entity do
       end
 
       it 'returns multiple entities if called with a collection' do
-        representation = subject.represent(4.times.map { Object.new })
+        representation = subject.represent(Array.new(4) { Object.new })
         expect(representation).to be_kind_of Array
         expect(representation.size).to eq(4)
         expect(representation.reject { |r| r.is_a?(subject) }).to be_empty
       end
 
       it 'adds the collection: true option if called with a collection' do
-        representation = subject.represent(4.times.map { Object.new })
+        representation = subject.represent(Array.new(4) { Object.new })
         representation.each { |r| expect(r.options[:collection]).to be true }
       end
 
@@ -616,7 +825,7 @@ describe Grape::Entity do
 
       it 'returns a serialized array of hashes of multiple objects if serializable: true' do
         subject.expose(:awesome) { |_| true }
-        representation = subject.represent(2.times.map { Object.new }, serializable: true)
+        representation = subject.represent(Array.new(2) { Object.new }, serializable: true)
         expect(representation).to eq([{ awesome: true }, { awesome: true }])
       end
 
@@ -642,7 +851,7 @@ describe Grape::Entity do
       context 'with specified fields' do
         it 'returns only specified fields with only option' do
           subject.expose(:id, :name, :phone)
-          representation = subject.represent(OpenStruct.new, only: [:id, :name], serializable: true)
+          representation = subject.represent(OpenStruct.new, only: %i[id name], serializable: true)
           expect(representation).to eq(id: nil, name: nil)
         end
 
@@ -655,7 +864,7 @@ describe Grape::Entity do
         it 'returns only fields specified in the only option and not specified in the except option' do
           subject.expose(:id, :name, :phone)
           representation = subject.represent(OpenStruct.new,
-                                             only: [:name, :phone],
+                                             only: %i[name phone],
                                              except: [:phone], serializable: true)
           expect(representation).to eq(name: nil)
         end
@@ -725,7 +934,7 @@ describe Grape::Entity do
           subject.expose(:id, :name, :phone)
           subject.expose(:user, using: user_entity)
 
-          representation = subject.represent(OpenStruct.new(user: {}), only: [:id, :name, { user: [:name, :email] }], serializable: true)
+          representation = subject.represent(OpenStruct.new(user: {}), only: [:id, :name, { user: %i[name email] }], serializable: true)
           expect(representation).to eq(id: nil, name: nil, user: { name: nil, email: nil })
         end
 
@@ -748,7 +957,7 @@ describe Grape::Entity do
           subject.expose(:user, using: user_entity)
 
           representation = subject.represent(OpenStruct.new(user: {}),
-                                             only: [:id, :name, :phone, user: [:id, :name, :email]],
+                                             only: [:id, :name, :phone, user: %i[id name email]],
                                              except: [:phone, { user: [:id] }], serializable: true)
           expect(representation).to eq(id: nil, name: nil, user: { name: nil, email: nil })
         end
@@ -760,7 +969,7 @@ describe Grape::Entity do
               subject.expose(:name)
             end
 
-            representation = subject.represent(OpenStruct.new, condition: true, only: [:id, :name], serializable: true)
+            representation = subject.represent(OpenStruct.new, condition: true, only: %i[id name], serializable: true)
             expect(representation).to eq(id: nil, name: nil)
           end
 
@@ -770,7 +979,7 @@ describe Grape::Entity do
               subject.expose(:name, :mobile_phone)
             end
 
-            representation = subject.represent(OpenStruct.new, condition: true, except: [:phone, :mobile_phone], serializable: true)
+            representation = subject.represent(OpenStruct.new, condition: true, except: %i[phone mobile_phone], serializable: true)
             expect(representation).to eq(id: nil, name: nil)
           end
 
@@ -852,7 +1061,7 @@ describe Grape::Entity do
             subject.expose(:id)
             subject.expose(:name, as: :title)
 
-            representation = subject.represent(OpenStruct.new, condition: true, only: [:id, :title], serializable: true)
+            representation = subject.represent(OpenStruct.new, condition: true, only: %i[id title], serializable: true)
             expect(representation).to eq(id: nil, title: nil)
           end
 
@@ -879,7 +1088,7 @@ describe Grape::Entity do
             subject.expose(:nephew, using: nephew_entity)
 
             representation = subject.represent(OpenStruct.new(user: {}),
-                                               only: [:id, :name, :user], except: [:nephew], serializable: true)
+                                               only: %i[id name user], except: [:nephew], serializable: true)
             expect(representation).to eq(id: nil, name: nil, user: { id: nil, name: nil, email: nil })
           end
         end
@@ -891,7 +1100,7 @@ describe Grape::Entity do
         subject.present_collection true
         subject.expose :items
 
-        representation = subject.represent(4.times.map { Object.new })
+        representation = subject.represent(Array.new(4) { Object.new })
         expect(representation).to be_kind_of(subject)
         expect(representation.object).to be_kind_of(Hash)
         expect(representation.object).to have_key :items
@@ -903,8 +1112,9 @@ describe Grape::Entity do
         subject.present_collection true, :my_items
         subject.expose :my_items
 
-        representation = subject.represent(4.times.map { Object.new }, serializable: true)
+        representation = subject.represent(Array.new(4) { Object.new }, serializable: true)
         expect(representation).to be_kind_of(Grape::Entity::Exposure::NestingExposure::OutputBuilder)
+        expect(representation).to be_kind_of(Hash)
         expect(representation).to have_key :my_items
         expect(representation[:my_items]).to be_kind_of Array
         expect(representation[:my_items].size).to be 4
@@ -928,7 +1138,7 @@ describe Grape::Entity do
 
         context 'with an array of objects' do
           it 'allows a root element name to be specified' do
-            representation = subject.represent(4.times.map { Object.new })
+            representation = subject.represent(Array.new(4) { Object.new })
             expect(representation).to be_kind_of Hash
             expect(representation).to have_key 'things'
             expect(representation['things']).to be_kind_of Array
@@ -939,13 +1149,13 @@ describe Grape::Entity do
 
         context 'it can be overridden' do
           it 'can be disabled' do
-            representation = subject.represent(4.times.map { Object.new }, root: false)
+            representation = subject.represent(Array.new(4) { Object.new }, root: false)
             expect(representation).to be_kind_of Array
             expect(representation.size).to eq 4
             expect(representation.reject { |r| r.is_a?(subject) }).to be_empty
           end
           it 'can use a different name' do
-            representation = subject.represent(4.times.map { Object.new }, root: 'others')
+            representation = subject.represent(Array.new(4) { Object.new }, root: 'others')
             expect(representation).to be_kind_of Hash
             expect(representation).to have_key 'others'
             expect(representation['others']).to be_kind_of Array
@@ -971,7 +1181,7 @@ describe Grape::Entity do
 
         context 'with an array of objects' do
           it 'allows a root element name to be specified' do
-            representation = subject.represent(4.times.map { Object.new })
+            representation = subject.represent(Array.new(4) { Object.new })
             expect(representation).to be_kind_of Array
             expect(representation.size).to eq 4
             expect(representation.reject { |r| r.is_a?(subject) }).to be_empty
@@ -992,7 +1202,7 @@ describe Grape::Entity do
 
         context 'with an array of objects' do
           it 'allows a root element name to be specified' do
-            representation = subject.represent(4.times.map { Object.new })
+            representation = subject.represent(Array.new(4) { Object.new })
             expect(representation).to be_kind_of Hash
             expect(representation).to have_key('things')
             expect(representation['things']).to be_kind_of Array
@@ -1017,7 +1227,7 @@ describe Grape::Entity do
 
         it 'inherits array root root' do
           child_class = Class.new(subject)
-          representation = child_class.represent(4.times.map { Object.new })
+          representation = child_class.represent(Array.new(4) { Object.new })
           expect(representation).to be_kind_of Hash
           expect(representation).to have_key('things')
           expect(representation['things']).to be_kind_of Array
@@ -1329,12 +1539,28 @@ describe Grape::Entity do
         it 'allows to pass different :only and :except params using the same instance' do
           fresh_class.expose :a, :b, :c
           presenter = fresh_class.new(a: 1, b: 2, c: 3)
-          expect(presenter.serializable_hash(only: [:a, :b])).to eq(a: 1, b: 2)
-          expect(presenter.serializable_hash(only: [:b, :c])).to eq(b: 2, c: 3)
+          expect(presenter.serializable_hash(only: %i[a b])).to eq(a: 1, b: 2)
+          expect(presenter.serializable_hash(only: %i[b c])).to eq(b: 2, c: 3)
         end
       end
     end
 
+    describe '#inspect' do
+      before do
+        fresh_class.class_eval do
+          expose :name, :email
+        end
+      end
+
+      it 'does not serialize delegator or options' do
+        data = subject.inspect
+        expect(data).to include 'name='
+        expect(data).to include 'email='
+        expect(data).to_not include '@options'
+        expect(data).to_not include '@delegator'
+      end
+    end
+
     describe '#value_for' do
       before do
         fresh_class.class_eval do
@@ -1523,12 +1749,13 @@ describe Grape::Entity do
         expect(subject.value_for(:fantasies)).to eq ['Nessy', 'Double Rainbows', 'Unicorns']
       end
 
-      it 'tries instance methods on the entity first' do
+      context 'delegate_attribute' do
         module EntitySpec
           class DelegatingEntity < Grape::Entity
             root 'friends', 'friend'
             expose :name
             expose :email
+            expose :system
 
             private
 
@@ -1538,14 +1765,33 @@ describe Grape::Entity do
           end
         end
 
-        friend = double('Friend', name: 'joe', email: 'joe@example.com')
-        rep = EntitySpec::DelegatingEntity.new(friend)
-        expect(rep.value_for(:name)).to eq 'cooler name'
-        expect(rep.value_for(:email)).to eq 'joe@example.com'
+        it 'tries instance methods on the entity first' do
+          friend = double('Friend', name: 'joe', email: 'joe@example.com')
+          rep = EntitySpec::DelegatingEntity.new(friend)
+          expect(rep.value_for(:name)).to eq 'cooler name'
+          expect(rep.value_for(:email)).to eq 'joe@example.com'
 
-        another_friend = double('Friend', email: 'joe@example.com')
-        rep = EntitySpec::DelegatingEntity.new(another_friend)
-        expect(rep.value_for(:name)).to eq 'cooler name'
+          another_friend = double('Friend', email: 'joe@example.com')
+          rep = EntitySpec::DelegatingEntity.new(another_friend)
+          expect(rep.value_for(:name)).to eq 'cooler name'
+        end
+
+        it 'does not delegate Kernel methods' do
+          foo = double 'Foo', system: 'System'
+          rep = EntitySpec::DelegatingEntity.new foo
+          expect(rep.value_for(:system)).to eq 'System'
+        end
+
+        module EntitySpec
+          class DerivedEntity < DelegatingEntity
+          end
+        end
+
+        it 'derived entity get methods from base entity' do
+          foo = double 'Foo', name: 'joe'
+          rep = EntitySpec::DerivedEntity.new foo
+          expect(rep.value_for(:name)).to eq 'cooler name'
+        end
       end
 
       context 'using' do
