@@ -1,3 +1,14 @@
+# frozen_string_literal: true
+
+require 'simplecov'
+require 'coveralls'
+
+SimpleCov.start do
+  add_filter 'spec/'
+end
+
+Coveralls.wear!
+
 $LOAD_PATH.unshift(File.dirname(__FILE__))
 $LOAD_PATH.unshift(File.join(File.dirname(__FILE__), '..', 'lib'))
 $LOAD_PATH.unshift(File.join(File.dirname(__FILE__), 'support'))
@@ -7,6 +18,4 @@ require 'bundler'
 
 Bundler.require :default, :test
 
-require 'pry'
-
 RSpec.configure(&:raise_errors_for_deprecations!)
